import Sofa
import math
import os
import sys
import csv
import yaml

__file = __file__.replace('\\', '/') # windows


def createScene(rootNode):
    
    springVerification(rootNode)
    return 0



class springVerification(Sofa.PythonScriptController):


    def __init__(self, rootNode):
        if not os.path.isdir('data'):
            os.mkdir('data')

        # configuration
        self.solver_type = "Newton"   # options are "Euler", "Newton", and "Symplectic"
        self.linear_solver_type = "CG"   # options are "CG" and "Pardiso"
        self.impact_type = "ControlPoint"    # options are "ControlPoint", "LinearForce"
        self.external_points = "0.0 0.001 0.0" # "0.0 0.02 0.0" # "0.0 0.015 0.015"
        self.control_spring_stiffness = 100000000
        self.linear_force_times = "0.0 1.0 2.0 5.0 10.0 15.0 20.0 25.0 30.0 35.0 40.0 45.0 60.0 80.0 100.0"
        self.linear_force_forces = "0.0 -0.0015 0.0 0.0 -0.5 0.0 0.0 -1.0 0.0 0.0 -2.6 0.0 0.0 -37.5 0.0 0.0 -108.4 0.0 0.0 -309.7 0.0 0.0 -881.3 0.0 0.0 -2504.1 0.0 0.0 -7111.2 0.0 0.0 -20191.0 0.0 0.0 -57325.7 0.0 0.0 -162753.8 0.0 0.0 -1627530.8 0.0 0.0 -16275300.8 0.0"

        # spring parameters configuration
        self.spring_type = "PolynomialRestShape"    # options are "Polynomial" and "PolynomialRestShape"
        self.initial_length = 0.02 # 0.021213203
        self.polynomial_stiffness = [30.0, -500.0, 3000.0] # [8.0, 15.0, 20.0]
        self.polynomial_degree = [3]

        if self.linear_solver_type == 'Pardiso':
            rootNode.createObject('RequiredPlugin', name='Pardiso', pluginName='SofaPardisoSolver')
        self.createGraph(rootNode)
        return None



    def createGraph(self, rootNode):
        # rootNode
        self.rootNode = rootNode
        rootNode.findData('dt').value = 0.01
        rootNode.findData('gravity').value = "0 0 0"
        rootNode.createObject('VisualStyle', displayFlags='showBehaviorModels showForceFields showCollisionModels hideVisual')

        # rootNode/fixedNodes
        fixNode = rootNode.createChild('fixNode')
        fixNode.createObject('MechanicalObject', template='Vec3d', name='fixElements', showObject='true', position=self.external_points,  drawMode='1', showObjectScale='0.001')

        # rootNode/externalImpact
        self.iterations = 0
        if self.impact_type == 'ControlPoint':
            dotNode = rootNode.createChild('dotNode')
            self.impactPoint = dotNode.createObject('MechanicalObject', template='Vec3d', name='dot', showObject='true', position='0.0 0.0 0.0',  drawMode='1', showObjectScale='0.002')
            dotNode.createObject('BoxROI', name='dotBounds', box='-0.02 -0.02 -0.02 0.02 0.02 0.02', doUpdate='0')
            dotNode.createObject('Monitor', name='toolMonitor', template='Vec3d', indices='@dotBounds.indices', ExportPositions='1', fileName='data/brickCPimpact')

        # solvers
        simuNode = rootNode.createChild('simuNode')
        if self.solver_type == 'Euler':
            simuNode.createObject('EulerImplicitSolver', rayleighStiffness=0.1, rayleighMass=0.1)
        elif self.solver_type == 'Newton':
            simuNode.createObject('StaticSolver', name="NewtonStatic", printLog="0", correction_tolerance_threshold="1e-8", residual_tolerance_threshold="1e-8", should_diverge_when_residual_is_growing="1", newton_iterations="100")
        else:
            print 'Unknown solver type!'

        if self.linear_solver_type == "Pardiso":
            simuNode.createObject('SparsePARDISOSolver', name='LDLsolver', verbose='0', symmetric='2', exportDataToFolder='', printLog="1")
        elif self.linear_solver_type == "CG":
            simuNode.createObject('CGLinearSolver', iterations="200", tolerance="1e-12", threshold="1e-12")
        else:
            print 'Unknown linear solver type!'


        # object
        simuNode.createObject('MechanicalObject', position='0.0 0.0 0.0', name='Volume', showObject='true', drawMode='1', showObjectScale='0.005')
        simuNode.createObject('UniformMass', totalMass=0.2)
        simuNode.createObject('BoxROI', name='boundBox', box='-0.02 -0.02 -0.02 0.02 0.02 0.02', doUpdate='0')
        simuNode.createObject('Monitor', name='ObservationMonitor', indices='@boundBox.indices', fileName = 'data/pointForceField', ExportPositions='1', ExportVelocities='0', ExportForces='1')


        # spring attachment
        if self.spring_type == 'PolynomialRestShape':
            simuNode.createObject('PolynomialRestShapeSpringsForceField', polynomialStiffness=self.polynomial_stiffness, showIndicesScale='0', listening="1", printLog="0", points='@boundBox.indices', initialLength=self.initial_length, polynomialDegree=self.polynomial_degree, external_rest_shape='@../fixNode/fixElements', smoothShift="0.0001", smoothScale='10000000', external_points='0', drawSpring='1')
        elif self.spring_type == 'Polynomial':
            simuNode.createObject('PolynomialSpringsForceField', polynomialStiffness=self.polynomial_stiffness, listening="1", printLog="0", object1='@Volume', firstObjectPoints='@boundBox.indices', polynomialDegree=self.polynomial_degree, object2='@../fixNode/fixElements', secondObjectPoints='0', drawMode='0')
        else:
            print 'Unknown type of attachments'


        # control force or attachment
        if self.impact_type == 'ControlPoint':
            simuNode.createObject('RestShapeSpringsForceField', name='Springs', stiffness=self.control_spring_stiffness, angularStiffness='1', external_rest_shape='@../dotNode/dot', points='@boundBox.indices', drawSpring='1')
        elif self.impact_type == 'LinearForce':
            self.controlForce = simuNode.createObject('LinearForceField', name='impactForce', points='@boundBox.indices', times=self.linear_force_times, forces=self.linear_force_forces)

        return 0;


    def onEndAnimationStep(self, deltaTime):

        if self.impact_type == 'ControlPoint' and self.iterations < 1000000:
            position = self.impactPoint.findData('position').value
            position[0][1] = position[0][1] - 0.00002
            position[0][2] = position[0][2] - 0.00002
            self.impactPoint.findData('position').value = position
            self.iterations = self.iterations + 1

        if self.impact_type == 'LinearForce':
            time = self.rootNode.findData('time').value
            force = self.controlForce.findData('targetForce').value
            fil = open('data/forceData.txt', "a")
            fil.write(str(time) + " ")
            fil.write(str(force[0][0]) + " " + str(force[0][1]) + " " + str(force[0][2]) + "\n")
            fil.close()

        return 0;


    def onMouseButtonLeft(self, mouseX,mouseY,isPressed):
        ## usage e.g.
        #if isPressed : 
        #    print "Control+Left mouse button pressed at position "+str(mouseX)+", "+str(mouseY)
        return 0;

    def onKeyReleased(self, c):
        ## usage e.g.
        #if c=="A" :
        #    print "You released a"
        return 0;

    def initGraph(self, node):
        ## Please feel free to add an example for a simple usage in /home/ip/Work/sofa/MyPlugins/Optimus/scenes/assimStiffness//home/ip/Work/sofa/master/src/applications/plugins/SofaPython/scn2python.py
        return 0;

    def onKeyPressed(self, c):
        ## usage e.g.
        #if c=="A" :
        #    print "You pressed control+a"
        return 0;

    def onMouseWheel(self, mouseX,mouseY,wheelDelta):
        ## usage e.g.
        #if isPressed : 
        #    print "Control button pressed+mouse wheel turned at position "+str(mouseX)+", "+str(mouseY)+", wheel delta"+str(wheelDelta)
        return 0;

    def storeResetState(self):
        ## Please feel free to add an example for a simple usage in /home/ip/Work/sofa/MyPlugins/Optimus/scenes/assimStiffness//home/ip/Work/sofa/master/src/applications/plugins/SofaPython/scn2python.py
        return 0;

    def cleanup(self):
        ## Please feel free to add an example for a simple usage in /home/ip/Work/sofa/MyPlugins/Optimus/scenes/assimStiffness//home/ip/Work/sofa/master/src/applications/plugins/SofaPython/scn2python.py
        return 0;

    def onGUIEvent(self, strControlID,valueName,strValue):
        ## Please feel free to add an example for a simple usage in /home/ip/Work/sofa/MyPlugins/Optimus/scenes/assimStiffness//home/ip/Work/sofa/master/src/applications/plugins/SofaPython/scn2python.py
        return 0;

    def onLoaded(self, node):
        ## Please feel free to add an example for a simple usage in /home/ip/Work/sofa/MyPlugins/Optimus/scenes/assimStiffness//home/ip/Work/sofa/master/src/applications/plugins/SofaPython/scn2python.py
        return 0;

    def reset(self):
        ## Please feel free to add an example for a simple usage in /home/ip/Work/sofa/MyPlugins/Optimus/scenes/assimStiffness//home/ip/Work/sofa/master/src/applications/plugins/SofaPython/scn2python.py
        return 0;

    def onMouseButtonMiddle(self, mouseX,mouseY,isPressed):
        ## usage e.g.
        #if isPressed : 
        #    print "Control+Middle mouse button pressed at position "+str(mouseX)+", "+str(mouseY)
        return 0;

    def bwdInitGraph(self, node):
        ## Please feel free to add an example for a simple usage in /home/ip/Work/sofa/MyPlugins/Optimus/scenes/assimStiffness//home/ip/Work/sofa/master/src/applications/plugins/SofaPython/scn2python.py
        return 0;

    def onScriptEvent(self, senderNode, eventName,data):
        ## Please feel free to add an example for a simple usage in /home/ip/Work/sofa/MyPlugins/Optimus/scenes/assimStiffness//home/ip/Work/sofa/master/src/applications/plugins/SofaPython/scn2python.py
        return 0;

    def onMouseButtonRight(self, mouseX,mouseY,isPressed):
        ## usage e.g.
        #if isPressed : 
        #    print "Control+Right mouse button pressed at position "+str(mouseX)+", "+str(mouseY)
        return 0;

    def onBeginAnimationStep(self, deltaTime):
        ## Please feel free to add an example for a simple usage in /home/ip/Work/sofa/MyPlugins/Optimus/scenes/assimStiffness//home/ip/Work/sofa/master/src/applications/plugins/SofaPython/scn2python.py
        return 0;


